// ----- OPRS HEADER FILES -----
#include <opaque-pub.h>
#include <mp-pub.h>

class OPRSclient {

	private:
		bool mp_connection;
		int mp_socket_fd;
	
	public:
		OPRSclient();
		~OPRSclient();
		bool connectToHost(const std::string& name, const std::string &hostname, const int portn);
		void disconnectFromMP();
		void setDisconnected();
		std::pair<std::string, std::string> getMsgFromMP();
		void sendMsgToMPClient(const std::string &name, const std::string& msg);
		
};

	OPRSclient::OPRSclient() : mp_connection(false), mp_socket_fd(-1) {}
	OPRSclient::~OPRSclient() {}

	bool OPRSclient::connectToHost(const std::string& name, const std::string &hostname, const int portn) {
		if ((mp_socket_fd = external_register_to_the_mp_host_prot(name.c_str(), hostname.c_str(), portn, STRINGS_PT)) == -1) 
			mp_connection = false;
		else mp_connection = true;
		return mp_connection;
	}

	void OPRSclient::disconnectFromMP() {
		if (!mp_connection) return;
		close(mp_socket_fd);
		mp_socket_fd = -1;
		mp_connection = false;
	}

	std::pair<std::string, std::string> OPRSclient::getMsgFromMP() {
		fd_set readfds;
		int nfound; int max_fds;
		struct timeval zero_tv;
		std::string sender, message("NO_NEW_MESSAGE");
		if (!mp_connection) return make_pair("", std::string("NOT_CONNECTED"));
		FD_ZERO(&readfds); FD_SET(mp_socket_fd, &readfds);
		max_fds = mp_socket_fd + 1;
		zero_tv.tv_sec = 0; zero_tv.tv_usec = 0; 
		nfound = select(max_fds, &readfds, NULL, NULL, &zero_tv);
		if (nfound <= 0) return make_pair("", message);
		if(FD_ISSET(mp_socket_fd, &readfds)){
			int length;
			char* senderPtr = read_string_from_socket(mp_socket_fd, &length);
			if (senderPtr == NULL) {
				mp_connection = false;
				return make_pair("", std::string("NOT_CONNECTED"));
			}
			sender = std::string(senderPtr);
			char* messagePtr = read_string_from_socket(mp_socket_fd, &length);
			if (messagePtr == NULL) {
				mp_connection = false;
				return make_pair("", std::string("NOT_CONNECTED"));
			}
			return make_pair(sender, std::string(messagePtr));
		}
		return make_pair("", message);
	}

	void OPRSclient::sendMsgToMPClient(const std::string &name, const std::string& msg) {
		if (!mp_connection) return;
		char* Msg = strdup(msg.c_str());
		send_message_string(Msg, const_cast<char*>(name.c_str()));
	}

